# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
import os
import sys
from recommonmark.transform import AutoStructify
sys.path.insert(0, os.path.abspath('../src/hightea/plotting/'))


# -- Project information -----------------------------------------------------

project = 'HighTEA-plotting'
copyright = '2021-22 Hightea Collaboration'
author = 'Hightea Collaboration'

# The full version, including alpha/beta/rc tags
# release = ''


# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.doctest',
    'sphinx.ext.coverage',
    'sphinx.ext.napoleon',
    'sphinx.ext.intersphinx',
    'recommonmark'
]

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# Markdown configuration

# The suffix(es) of source filenames.
# You can specify multiple suffix as a list of string:
#
source_suffix = {
    '.rst': 'restructuredtext',
    '.txt': 'markdown',
    '.md': 'markdown',
}

autosectionlabel_prefix_document = True
# Allow to embed rst syntax in  markdown files.
enable_eval_rst = True

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = []


# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
# html_theme = 'classic'
html_theme = 'sphinx_rtd_theme'

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']

# -- Intersphinx  -------------------------------------------------------------

intersphinx_mapping = {'python': ('https://docs.python.org/3', None)}

# -- Doctest ------------------------------------------------------------------
#

doctest_path = [os.path.abspath('examples')]

# -- Autodoc ------------------------------------------------------------------
#
autodoc_default_options = {
    'members': True,
    'member-order': 'bysource',
    'special-members': '__init__',
    'undoc-members': True,
}

# Adapted this from
# https://github.com/readthedocs/recommonmark/blob/ddd56e7717e9745f11300059e4268e204138a6b1/docs/conf.py
# app setup hook
def setup(app):
    app.add_config_value('recommonmark_config', {
        'enable_eval_rst': True,
    }, True)
    app.add_transform(AutoStructify)



from .plotting import plot
from .run import Run
import numpy as np


def get_31variation(num,denom):
    """31 scale variation for ratio plot

    Parameters
    ----------
    num : Run
        Run in the numerator.

    denom : Run
        Run in the denomerator.

    Result
    ------
    Run
        Metadata is taken from the numerator.
    """
    setups = [(1,1),(2,2),(0,0),(2,1),(1,2),(0,1),(1,0)]
    variation = [ (i,j) for i in range(7) for j in range(7)
                if not(max(*setups[i],*setups[j]) - min(*setups[i],*setups[j]) == 2) ]
    ratio_runs = [ num[i]/denom[j] for i,j in variation ]
    ratio = Run(bins=num.bins)
    ratio.values = np.transpose([r.v() for r in ratio_runs])
    ratio.errors = np.transpose([r.e() for r in ratio_runs])
    ratio.update_info(num)
    return ratio




import pytest
from math import isclose
from src.hightea.plotting.run import Run
import numpy as np
from copy import deepcopy
from pathlib import Path
import os
import shutil

def test_RunClass_create():
    run = Run()

#################
#  I/O methods  #
#################

def test_loading_dict():
    run = Run()
    run.load({'histogram': [
        {'edges':[[0, 1]],'mean':[0], 'error':[0]},
        {'edges':[[1, 2]],'mean':[1], 'error':[0]},
        {'edges':[[2, 3]],'mean':[2], 'error':[0]},
        ]})
    assert((run.edges[0] == np.array([0,1,2,3])).all())
    run.load({'histogram': [
        {'edges':[{'min_value':0,'max_value':1}],'mean':[0], 'error':[0]},
        {'edges':[{'min_value':1,'max_value':2}],'mean':[1], 'error':[0]},
        {'edges':[{'min_value':2,'max_value':3}],'mean':[2], 'error':[0]},
        ]})
    assert((run.edges[0] == np.array([0,1,2,3])).all())
    assert((run.v() == np.array([0,1,2])).all())
    # with pytest.warns(UserWarning):
    run.load({'histogram': [
        {'edges':[{'min_value':0,'max_value':1}],'mean':0},
        {'edges':[{'min_value':1,'max_value':2}],'mean':1},
        {'edges':[{'min_value':2,'max_value':3}],'mean':2},
        ]})
    assert((run.v() == np.array([0,1,2])).all())

def test_loading_json():
    run = Run()
    run.load('tests/input/simple1d.json')
    assert run.bins[0] == [[0,1]]
    assert run.is_differential()

def test_dumping_json():
    run = Run.seq((4,5),nsetups=5)
    wdir = 'tests/tmp'
    if not os.path.isdir(wdir):
        os.makedirs(wdir)
    run.to_json(Path(wdir,'run.json'),verbose=False)
    new = Run(Path(wdir,'run-0.json'))
    assert new.nsetups() == 1
    run.to_json(Path(wdir,'run.json'),verbose=False,combined=True)
    new = Run(Path(wdir,'run.json'))
    assert new.nsetups() == run.nsetups()
    shutil.rmtree(wdir)

def test_loading_hepdata():
    run = Run()
    run.load('tests/input/HEPData-ins1610623-v1-Table_7.csv')
    assert run.dim() == 1
    assert isclose(run.edges[0][-1], 2.4)

def test_makehistogram_json():
    run = Run('tests/input/simple2d.json')
    newrun = deepcopy(run)
    run.make_histogramlike()
    run.make_differential()
    assert(run == newrun)

##############################
#  bin to edges conversions  #
##############################

def test_convert_to_edges_1d():
    bins = [[[0,1]],[[1,2]],[[2,3]]]
    result = Run.convert_to_edges(bins)
    expected = [np.array([0,1,2,3])]
    for b,e in zip(result, expected):
        assert((b == e).all())

def test_convert_to_edges_2d():
    bins = [[[0,1],[0,1]],[[0,1],[1,2]],[[1,2],[0,1]],[[1,2],[1,2]]]
    result = Run.convert_to_edges(bins)
    expected = [np.array([0,1,2]),np.array([0,1,2])]
    for b,e in zip(result, expected):
        assert((b == e).all())

    bins = [[[0,1],[0,1]],[[0,1],[1,2]]]
    result = Run.convert_to_edges(bins)
    expected = [np.array([0,1]),np.array([0,1,2])]
    for b,e in zip(result, expected):
        assert((b == e).all())

def test_convert_to_bins_1d():
    edges = [np.array([0,1,2,3])]
    result = Run.convert_to_bins(edges)
    expected = [[[0,1]],[[1,2]],[[2,3]]]
    assert(np.array_equal(result, expected))

def test_convert_to_bins_2d():
    edges = [np.array([0,1,3]),np.array([1,2])]
    result = Run.convert_to_bins(edges)
    expected = [[[0,1],[1,2]], [[1,3],[1,2]]]
    assert(np.array_equal(result, expected))

#####################
#  slicing methods  #
#####################

def test_slicing_scale():
    run = Run.random((2,3),4)
    assert((run[1:].nsetups()) == 3)
    assert((run[0].nsetups()) == 1)

def test_random():
    shape = [2,3,1]
    run = Run.random(shape,3)
    assert(run.dimensions() == shape)
    expected = [np.array([0,1,2]), np.array([0,1,2,3]), np.array([0,1])]
    for r,e in zip(run.edges, expected):
        assert((r == e).all())
    assert(run.values.shape == (len(run.bins),3))
    assert(run.errors.shape == (len(run.bins),3))

def test_zoom():
    # TODO: update test for slices at other dimensions
    run = Run.seq((2,3),2)
    run.edges = [np.array([-5,-3,-1]), np.array(list(range(4)))]
    run2 = run.zoom(-2)
    assert((run2.values[:,0] == np.array([3,4,5])).all())
    run2 = run.zoom(line=1)
    assert((run2.values[:,0] == np.array([3,4,5])).all())

#########################
#  ratio and rescaling  #
#########################

def test_div():
    run = Run.random((2,3),3)
    divrun = run[:]
    newrun = divrun / run
    assert(np.isclose(newrun.values.flatten(),np.ones(2*3*3)).all())

######################
#  Passing metadata  #
######################

def test_name():
    run = Run('tests/input/simple2d.json', name='testname')
    assert(run.name == 'testname')

##########################
#  Arithmetic operators  #
##########################

def test_eq():
    run = Run.random((2,3),3)
    new = run.minicopy()
    # TODO revise this test
    assert(new == run)

def test_add():
    run = Run.random((2,3),nsetups=3)
    zeros = Run.full((2,3),nsetups=3)
    new = zeros + run
    assert(new == run)
    assert(2 - 2 + zeros + 1 - 1 == zeros)

def test_muldiv():
    old = Run.random((2,3),nsetups=3)
    new = deepcopy(old)
    run = Run.random((2,3),nsetups=3)
    new *= run*.3
    new *= .3*run
    new /= .01*run
    new /= 9*run
    assert(np.isclose(new.values,old.values).all())

def test_mergebins():
    run = Run.seq((5,),nsetups=3)
    newrun = run.mergebins(pos=(1,4))
    assert newrun.edges[0][2] == 4
    assert np.isclose(newrun.values[0][0], run.values[0][0])
    assert np.isclose(newrun.values[1][0], 6.)
    assert np.isclose(newrun.errors[1][0], np.sqrt(.1**2+.2**2+.3**2))

    run.edges = [list(range(5,11))]
    newrun = run.mergebins(values=(5,8))
    assert np.isclose(newrun.edges[0][1], 8)




import src.hightea.plotting.plotting as hyt
from src.hightea.plotting.stripper import convert_to_Run, load_to_Run

def test_plot():
    hyt.plot('tests/input/simple1d.json', show=False)

def test_plot_lists():
    hyt.plot(['tests/input/simple1d.json']*2, show=False)

def test_ratioPlot():
    hyt.plot(load_to_Run('tests/input/ppWpjs_0_nnlo.xml'),
             load_to_Run('tests/input/ppWpjs_T_nnlo.xml'),
             show=False, ratio=0)

def test_hepdata():
    hyt.plot('tests/input/HEPData-ins1610623-v1-Table_7.csv', show=False)




import pytest
from math import isclose
import numpy as np
import src.hightea.plotting as hyt
from src.hightea.plotting.stripper import convert_to_Run, load_to_Run
from src.hightea.plotting._MeasurementTools  import MeasurementTools


def test_loading_xml_1d():
    """Load 1d observable from stripper xml"""
    mt = MeasurementTools()
    mt.loadxml('tests/input/test.xml')
    run = convert_to_Run(mt, 0)
    assert(run.bins[0] == [[-0.5,0.5]])
    assert(len(run.edges) == 1)
    assert(len(run.bins[0]) == 1)
    assert(isinstance(run.edges[0], list))
    newrun = load_to_Run('tests/input/test.xml')
    assert(newrun.bins[0] == [[-0.5,0.5]])


def test_loading_xml_2d():
    """Load 2d observable from stripper xml"""
    mt = MeasurementTools()
    mt.loadxml('tests/input/test.xml')
    run = convert_to_Run(mt, obs=1)
    assert(len(run.edges) == 2)
    assert(len(run.bins[0]) == 2)
    assert(len(run.values.shape) == 2)
    assert(isinstance(run.edges[0], list))


def test_plot_xml_1d():
    """Trivial plotting"""
    run = load_to_Run('tests/input/test.xml')
    hyt.plot(run, show=False)

def test_xsec():
    """Xsection list and list of scales should have matching length"""
    run = load_to_Run('tests/input/test.xml')
    assert(run.xsec.shape[1] == run.values.shape[1])

def test_loadObservables():
    """Load observable by name"""
    filename = 'tests/input/test.xml'
    mt = MeasurementTools()
    mt.loadxml(filename)
    expected = mt.extractObservables(0)[0]
    run = load_to_Run(filename, obs='total cross section')
    assert(run.info.get('obs') == expected)
    with pytest.warns(UserWarning):
        run = load_to_Run(filename, obs='transverse momentum')
        assert(not(run.info.get('obs') == expected))
    with pytest.raises(Exception):
        run = load_to_Run(filename, obs='#@!')

